// Copyright (c) 2005 - 2017 Settlers Freaks (sf-team at siedler25.org)
//
// This file is part of Return To The Roots.
//
// Return To The Roots is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// Return To The Roots is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Return To The Roots. If not, see <http://www.gnu.org/licenses/>.

#ifndef PostBox_h__
#define PostBox_h__

#include <boost/array.hpp>
#include <boost/function.hpp>

class PostMsg;

class PostBox
{
public:
    typedef boost::function<void(const PostMsg&, unsigned)> NewMsgCallback;
    typedef boost::function<void(unsigned)> MsgDeletedCallback;

    PostBox();
    ~PostBox();

    /// Add the message, possibly deleting the oldest message
    void AddMsg(const PostMsg* msg);
    /// Delete the message. Return true, when it existed
    bool DeleteMsg(const PostMsg* msg);
    /// Delete the message by its index. Return true, if it was valid
    /// Oldest message is at index 0
    bool DeleteMsg(unsigned idx);
    void Clear();
    unsigned GetNumMsgs() const { return numMessages; }
    static BOOST_CONSTEXPR unsigned GetMaxMsgs() { return MAX_MESSAGES; }
    /// Get message by index or NULL if invalid index
    /// Oldest message is at index 0
    const PostMsg* GetMsg(unsigned idx) const;
    /// Set callback that receives new message and message count everytime a message is added
    void ObserveNewMsg(const NewMsgCallback& callback) { evNewMsg = callback; }
    /// Set callback that receives new message count everytime a message is deleted
    void ObserveDeletedMsg(const MsgDeletedCallback& callback) { evDelMsg = callback; }

    /// Sets the current goal as shown in the post window
    void SetCurrentMissionGoal(const std::string& goal);
    std::string GetCurrentMissionGoal() const;

private:
    BOOST_STATIC_CONSTEXPR unsigned MAX_MESSAGES = 20;
    bool DeleteMsg(unsigned idx, bool notify);
    boost::array<const PostMsg*, MAX_MESSAGES> messages;
    unsigned numMessages;
    /// Current mission goal. Shown as a special message
    std::string currentMissionGoal;
    NewMsgCallback evNewMsg;
    MsgDeletedCallback evDelMsg;
};

#endif // PostBox_h__
